@@ -9,18 +9,19 @@ use feature qw(say);
 use utf8;
 use Data::Dumper;
 use JSON::XS;
-use Time::HiRes qw(gettimeofday);
-use POSIX qw(strftime);
+use Time::HiRes  qw(gettimeofday);
+use POSIX        qw(strftime);
 use Scalar::Util qw(looks_like_number);
-use List::Util qw(first);
-use Cwd qw(cwd);
+use List::Util   qw(first);
+use Cwd          qw(cwd);
 use Sys::Hostname;
 use Convert::Pheno::SQLite;
-binmode STDOUT, ':encoding(utf-8)';
+use Convert::Pheno::Default qw(get_defaults);
 use Exporter 'import';
 our @EXPORT =
-  qw(map_ethnicity map_ontology dotify_and_coerce_number iso8601_time _map2iso8601 map_reference_range map_age_range map2redcap_dict map2ohdsi convert2boolean find_age randStr map_operator_concept_id map_info_field map_omop_visit_occurrence dot_date2iso remap_mapping_hash_term validate_format get_metaData get_info);
+  qw(map_ontology_term dotify_and_coerce_number iso8601_time _map2iso8601 map_reference_range map_reference_range_csv map_age_range map2redcap_dict map2ohdsi convert2boolean find_age randStr map_operator_concept_id map_info_field map_omop_visit_occurrence dot_date2iso validate_format get_metaData get_info);
 
+my $DEFAULT = get_defaults();
 use constant DEVEL_MODE => 0;
 
 # Global hash
@@ -32,16 +33,7 @@ my %seen = ();
 #############################
 #############################
 
-sub map_ethnicity {
-
-    my $str       = shift;
-    my %ethnicity = ( map { $_ => 'NCIT:C41261' } ( 'caucasian', 'white' ) );
-
-# 1, Caucasian | 2, Hispanic | 3, Asian | 4, African/African-American | 5, Indigenous American | 6, Mixed | 9, Other";
-    return { id => $ethnicity{ lc($str) }, label => $str };
-}
-
-sub map_ontology {
+sub map_ontology_term {
 
     # Most of the execution time goes to this subroutine
     # We will adopt two estragies to gain speed:
@@ -50,43 +42,55 @@ sub map_ontology {
 
     #return { id => 'dummy', label => 'dummy' } # test speed
 
+    # We will return quickly when nothing has to be done
+    my $query = $_[0]->{query};
+
+    # Skipping numbers
+    return $DEFAULT->{ontology_term} if looks_like_number($query);
+
+    # If the ontology term is an object we assume it comes
+    # from "terminology" property in the mapping file
+    return $query if ref $query eq 'HASH';
+
     # Checking for existance in %seen
-    my $tmp_query = $_[0]->{query};
-    say "Skipping searching for <$tmp_query> as it already exists"
-      if DEVEL_MODE && exists $seen{$tmp_query};
+    say "Skipping searching for <$query> as it already exists"
+      if DEVEL_MODE && exists $seen{$query};
 
     # return if terms has already been searched and exists
     # Not a big fan of global stuff...
     #  ¯\_(ツ)_/¯
     # Premature return
-    return $seen{$tmp_query} if exists $seen{$tmp_query};    # global
-
-    say "searching for <$tmp_query>" if DEVEL_MODE;
+    return $seen{$query} if exists $seen{$query};    # global
 
     # return something if we know 'a priori' that the query won't exist
-    #return { id => 'NCIT:NA000', label => $tmp_query } if $tmp_query =~ m/xx/;
+    #return { id => 'NCIT:NA000', label => $query } if $query =~ m/xx/;
 
     # Ok, now it's time to start the subroutine
     my $arg                       = shift;
     my $column                    = $arg->{column};
     my $ontology                  = $arg->{ontology};
     my $self                      = $arg->{self};
+    my $databases                 = $self->{databases};
     my $search                    = $self->{search};
     my $print_hidden_labels       = $self->{print_hidden_labels};
     my $text_similarity_method    = $self->{text_similarity_method};
     my $min_text_similarity_score = $self->{min_text_similarity_score};
 
+    # DEVEL_MODE
+    say "searching for query <$query> in ontology <$ontology>" if DEVEL_MODE;
+
     # Die if user wants OHDSI w/o flag -ohdsi-db
     die
-"Could not find the concept_id:<$tmp_query> in the provided <CONCEPT> table.\nPlease use the flag <--ohdsi-db> to enable searching at Athena-OHDSI database\n"
+"Could not find the concept_id:<$query> in the provided <CONCEPT> table.\nPlease use the flag <--ohdsi-db> to enable searching at Athena-OHDSI database\n"
       if ( $ontology eq 'ohdsi' && !$self->{ohdsi_db} );
 
     # Perform query
-    my ( $id, $label ) = get_ontology(
+    my ( $id, $label ) = get_ontology_terms(
         {
             sth_column_ref            => $self->{sth}{$ontology}{$column},
-            query                     => $tmp_query,
+            query                     => $query,
             ontology                  => $ontology,
+            databases                 => $databases,
             column                    => $column,
             search                    => $search,
             text_similarity_method    => $text_similarity_method,
@@ -95,11 +99,11 @@ sub map_ontology {
     );
 
     # Add result to global %seen
-    $seen{$tmp_query} = { id => $id, label => $label };    # global
+    $seen{$query} = { id => $id, label => $label };    # global
 
 # id and label come from <db> _label is the original string (can change on partial matches)
     return $print_hidden_labels
-      ? { id => $id, label => $label, _label => $tmp_query }
+      ? { id => $id, label => $label, _label => $query }
       : { id => $id, label => $label };
 }
 
@@ -159,6 +163,12 @@ sub map_reference_range {
     return $hashref;
 }
 
+sub map_reference_range_csv {
+    my ( $unit, $range ) = @_;
+    $range->{unit} = $unit;
+    return $range;
+}
+
 sub map_age_range {
 
     my $str = shift;
@@ -206,8 +216,8 @@ sub map2redcap_dict {
 sub map2ohdsi {
 
     my $arg = shift;
-    my ( $ohdsi_dic, $concept_id, $self ) =
-      ( $arg->{ohdsi_dic}, $arg->{concept_id}, $arg->{self} );
+    my ( $ohdsi_dict, $concept_id, $self ) =
+      ( $arg->{ohdsi_dict}, $arg->{concept_id}, $arg->{self} );
 
     #######################
     # OPTION A: <CONCEPT> #
@@ -217,10 +227,10 @@ sub map2ohdsi {
     # .. we are already searching in a hash
     # NB2: $concept_id is stringified by hash
     my ( $data, $id, $label, $vocabulary ) = ( (undef) x 4 );
-    if ( exists $ohdsi_dic->{$concept_id} ) {
-        $id         = $ohdsi_dic->{$concept_id}{concept_code};
-        $label      = $ohdsi_dic->{$concept_id}{concept_name};
-        $vocabulary = $ohdsi_dic->{$concept_id}{vocabulary_id};
+    if ( exists $ohdsi_dict->{$concept_id} ) {
+        $id         = $ohdsi_dict->{$concept_id}{concept_code};
+        $label      = $ohdsi_dict->{$concept_id}{concept_name};
+        $vocabulary = $ohdsi_dict->{$concept_id}{vocabulary_id};
         $data       = { id => qq($vocabulary:$id), label => $label };
     }
 
@@ -229,7 +239,7 @@ sub map2ohdsi {
     ######################
 
     else {
-        $data = map_ontology(
+        $data = map_ontology_term(
             {
                 query    => $concept_id,
                 column   => 'concept_id',
@@ -344,7 +354,7 @@ sub map_omop_visit_occurrence {
 
     my $arg                 = shift;
     my $self                = $arg->{self};
-    my $ohdsi_dic           = $arg->{ohdsi_dic};
+    my $ohdsi_dict          = $arg->{ohdsi_dict};
     my $person_id           = $arg->{person_id};
     my $visit_occurrence_id = $arg->{visit_occurrence_id};
     my $visit_occurrence    = $self->{visit_occurrence};
@@ -357,7 +367,7 @@ sub map_omop_visit_occurrence {
 # For instance, person_id = 1 has only visit_occurrence_id = 85, but on tables it has:
 # 82, 84, 42, 54, 41, 25, 76 and 81
 
-    # warn if we don't have $visit_occurrence_id in VISIT_OCURRENCE
+    # warn if we don't have $visit_occurrence_id in VISIT_OCCURRENCE
     unless ( exists $visit_occurrence->{$visit_occurrence_id} ) {
         warn
 "Sorry, but <visit_occurrence_id:$visit_occurrence_id> does not exist for <person_id:$person_id>\n"
@@ -372,7 +382,7 @@ sub map_omop_visit_occurrence {
 
     my $concept = map2ohdsi(
         {
-            ohdsi_dic  => $ohdsi_dic,
+            ohdsi_dict => $ohdsi_dict,
             concept_id => $hashref->{visit_concept_id},
             self       => $self
 
@@ -390,7 +400,7 @@ sub map_omop_visit_occurrence {
       ? $ad_hoc_44818517
       : map2ohdsi(
         {
-            ohdsi_dic  => $ohdsi_dic,
+            ohdsi_dict => $ohdsi_dict,
             concept_id => $hashref->{visit_type_concept_id},
             self       => $self
 
@@ -433,24 +443,6 @@ sub is_multidimensional {
     return ref shift ? 1 : 0;
 }
 
-sub remap_mapping_hash_term {
-
-    my ( $mapping_file_data, $term ) = @_;
-    my %hash_out = map {
-            $_ => exists $mapping_file_data->{$term}{$_}
-          ? $mapping_file_data->{$term}{$_}
-          : undef
-    } (qw/fields dictionary mapping selector/);
-    $hash_out{ontology} =
-      exists $mapping_file_data->{$term}{ontology}
-      ? $mapping_file_data->{$term}{ontology}
-      : $mapping_file_data->{project}{ontology};
-    $hash_out{routesOfAdministration} =
-      $mapping_file_data->{$term}{routesOfAdministration}
-      if $term eq 'treatments';
-    return \%hash_out;
-}
-
 sub validate_format {
 
     my ( $data, $format ) = @_;
@@ -473,11 +465,11 @@ sub get_info {
 
     my $self = shift;
 
-   # NB: Darwin does not have nproc to show #logical-cores, using sysctl instead
+    # Detecting the number of logical CPUs across different OSes
     my $os = $^O;
     chomp(
         my $ncpuhost =
-          lc($os) eq 'darwin' ? qx{/usr/sbin/sysctl -n hw.logicalcpu}
+          lc($os) eq 'darwin' || lc($os) eq 'freebsd' ? qx{sysctl -n hw.ncpu}
         : $os eq 'MSWin32' ? qx{wmic cpu get NumberOfLogicalProcessors}
         :                    qx{/usr/bin/nproc} // 1
     );
